@@ -14,15 +14,15 @@
 
 import errno
 import ipaddress
-import logging
 import os
-import platform
 import shutil
 import stat
 import subprocess
 
+import distro
 import jinja2
 from oslo_config import cfg
+from oslo_log import log as logging
 import six
 import webob
 from werkzeug import exceptions
@@ -41,17 +41,30 @@ j2_env = jinja2.Environment(autoescape=True, loader=jinja2.FileSystemLoader(
 
 class BaseOS(object):
 
+    PACKAGE_NAME_MAP = {}
+
     def __init__(self, os_name):
         self.os_name = os_name
 
+    @classmethod
+    def _get_subclasses(cls):
+        for subclass in cls.__subclasses__():
+            for sc in subclass._get_subclasses():
+                yield sc
+            yield subclass
+
     @classmethod
     def get_os_util(cls):
-        os_name = platform.linux_distribution(full_distribution_name=False)[0]
-        for subclass in BaseOS.__subclasses__():
+        os_name = distro.id()
+        for subclass in cls._get_subclasses():
             if subclass.is_os_name(os_name):
                 return subclass(os_name)
         raise octavia_exceptions.InvalidAmphoraOperatingSystem(os_name=os_name)
 
+    @classmethod
+    def _map_package_name(cls, package_name):
+        return cls.PACKAGE_NAME_MAP.get(package_name, package_name)
+
     def get_network_interface_file(self, interface):
         if CONF.amphora_agent.agent_server_network_file:
             return CONF.amphora_agent.agent_server_network_file
@@ -103,17 +116,15 @@ class BaseOS(object):
                 consts=consts,
                 interface=primary_interface,
                 vip=vip,
-                vip_ipv6=ip.version is 6,
-                # For ipv6 the netmask is already the prefix
-                prefix=(netmask if ip.version is 6
-                        else utils.netmask_to_prefix(netmask)),
+                vip_ipv6=ip.version == 6,
+                prefix=utils.netmask_to_prefix(netmask),
                 broadcast=broadcast,
                 netmask=netmask,
                 gateway=gateway,
                 network=utils.ip_netmask_to_cidr(vip, netmask),
                 mtu=mtu,
                 vrrp_ip=vrrp_ip,
-                vrrp_ipv6=vrrp_version is 6,
+                vrrp_ipv6=vrrp_version == 6,
                 host_routes=render_host_routes,
                 topology=CONF.controller_worker.loadbalancer_topology,
             )
@@ -136,12 +147,11 @@ class BaseOS(object):
 
         with os.fdopen(os.open(interface_file_path, flags, mode),
                        'w') as text_file:
-            text = self._generate_network_file_text(
-                netns_interface, fixed_ips, mtu, template_port)
+            text = self._generate_network_file_text(netns_interface, fixed_ips,
+                                                    mtu, template_port)
             text_file.write(text)
 
-    @classmethod
-    def _generate_network_file_text(cls, netns_interface, fixed_ips, mtu,
+    def _generate_network_file_text(self, netns_interface, fixed_ips, mtu,
                                     template_port):
         text = ''
         if fixed_ips is None:
@@ -157,15 +167,15 @@ class BaseOS(object):
                         cidr if isinstance(
                             cidr, six.text_type) else six.u(cidr))
                     broadcast = network.broadcast_address.exploded
-                    netmask = (network.prefixlen if ip.version is 6
+                    netmask = (network.prefixlen if ip.version == 6
                                else network.netmask.exploded)
-                    host_routes = cls.get_host_routes(fixed_ip)
+                    host_routes = self.get_host_routes(fixed_ip)
 
                 except ValueError:
                     return webob.Response(
                         json=dict(message="Invalid network IP"), status=400)
                 new_text = template_port.render(interface=netns_interface,
-                                                ipv6=ip.version is 6,
+                                                ipv6=ip.version == 6,
                                                 ip_address=ip.exploded,
                                                 broadcast=broadcast,
                                                 netmask=netmask,
@@ -174,8 +184,7 @@ class BaseOS(object):
                 text = '\n'.join([text, new_text])
         return text
 
-    @classmethod
-    def get_host_routes(cls, fixed_ip):
+    def get_host_routes(self, fixed_ip):
         host_routes = []
         for hr in fixed_ip.get('host_routes', []):
             network = ipaddress.ip_network(
@@ -185,30 +194,13 @@ class BaseOS(object):
             host_routes.append({'network': network, 'gw': hr['nexthop']})
         return host_routes
 
-    @classmethod
-    def _bring_if_up(cls, interface, what, flush=True):
+    def _bring_if_up(self, interface, what):
         # Note, we are not using pyroute2 for this as it is not /etc/netns
         # aware.
-        # Work around for bug:
-        # https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=845121
-        int_up = "ip netns exec {ns} ip link set {int} up".format(
-            ns=consts.AMPHORA_NAMESPACE, int=interface)
-        addr_flush = "ip netns exec {ns} ip addr flush {int}".format(
-            ns=consts.AMPHORA_NAMESPACE, int=interface)
-
         cmd = ("ip netns exec {ns} ifup {params}".format(
             ns=consts.AMPHORA_NAMESPACE, params=interface))
         try:
-            out = subprocess.check_output(int_up.split(),
-                                          stderr=subprocess.STDOUT)
-            LOG.debug(out)
-            if flush:
-                out = subprocess.check_output(addr_flush.split(),
-                                              stderr=subprocess.STDOUT)
-                LOG.debug(out)
-            out = subprocess.check_output(cmd.split(),
-                                          stderr=subprocess.STDOUT)
-            LOG.debug(out)
+            subprocess.check_output(cmd.split(), stderr=subprocess.STDOUT)
         except subprocess.CalledProcessError as e:
             LOG.error('Failed to ifup %s due to error: %s %s', interface, e,
                       e.output)
@@ -217,8 +209,7 @@ class BaseOS(object):
                     message='Error plugging {0}'.format(what),
                     details=e.output), status=500))
 
-    @classmethod
-    def _bring_if_down(cls, interface):
+    def _bring_if_down(self, interface):
         # Note, we are not using pyroute2 for this as it is not /etc/netns
         # aware.
         cmd = ("ip netns exec {ns} ifdown {params}".format(
@@ -229,14 +220,13 @@ class BaseOS(object):
             LOG.info('Ignoring failure to ifdown %s due to error: %s %s',
                      interface, e, e.output)
 
-    @classmethod
-    def bring_interfaces_up(cls, ip, primary_interface, secondary_interface):
-        cls._bring_if_down(primary_interface)
+    def bring_interfaces_up(self, ip, primary_interface, secondary_interface):
+        self._bring_if_down(primary_interface)
         if secondary_interface:
-            cls._bring_if_down(secondary_interface)
-        cls._bring_if_up(primary_interface, 'VIP')
+            self._bring_if_down(secondary_interface)
+        self._bring_if_up(primary_interface, 'VIP')
         if secondary_interface:
-            cls._bring_if_up(secondary_interface, 'VIP', flush=False)
+            self._bring_if_up(secondary_interface, 'VIP')
 
     def has_ifup_all(self):
         return True
@@ -249,10 +239,11 @@ class Ubuntu(BaseOS):
 
     @classmethod
     def is_os_name(cls, os_name):
-        return os_name in ['Ubuntu']
+        return os_name in ['ubuntu']
 
     def cmd_get_version_of_installed_package(self, package_name):
-        return "dpkg-query -W -f=${{Version}} {name}".format(name=package_name)
+        name = self._map_package_name(package_name)
+        return "dpkg-query -W -f=${{Version}} {name}".format(name=name)
 
     def get_network_interface_file(self, interface):
         if CONF.amphora_agent.agent_server_network_file:
@@ -330,14 +321,18 @@ class RH(BaseOS):
     ETH_X_ALIAS_VIP_CONF = 'rh_plug_vip_ethX_alias.conf.j2'
     ROUTE_ETH_X_CONF = 'rh_route_ethX.conf.j2'
     RULE_ETH_X_CONF = 'rh_rule_ethX.conf.j2'
+    # The reason of make them as jinja templates is the current scripts force
+    # to add the iptables, so leave it now for future extending if possible.
+    ETH_IFUP_LOCAL_SCRIPT = 'rh_plug_port_eth_ifup_local.conf.j2'
+    ETH_IFDOWN_LOCAL_SCRIPT = 'rh_plug_port_eth_ifdown_local.conf.j2'
 
     @classmethod
     def is_os_name(cls, os_name):
-        return os_name in ['fedora', 'redhat', 'centos']
+        return os_name in ['fedora', 'rhel']
 
     def cmd_get_version_of_installed_package(self, package_name):
-        return "rpm -q --queryformat %{{VERSION}} {name}".format(
-            name=package_name)
+        name = self._map_package_name(package_name)
+        return "rpm -q --queryformat %{{VERSION}} {name}".format(name=name)
 
     def get_network_interface_file(self, interface):
         if CONF.amphora_agent.agent_server_network_file:
@@ -432,6 +427,8 @@ class RH(BaseOS):
                 route_rules_interface_file_path, primary_interface,
                 render_host_routes, template_rules, gateway, vip, netmask)
 
+        self._write_ifup_ifdown_local_scripts_if_possible()
+
     def write_static_routes_interface_file(self, interface_file_path,
                                            interface, host_routes,
                                            template_routes, gateway,
@@ -483,16 +480,62 @@ class RH(BaseOS):
             self.write_static_routes_interface_file(
                 routes_interface_file_path, netns_interface,
                 host_routes, template_routes, None, None, None)
+            self._write_ifup_ifdown_local_scripts_if_possible()
 
-    @classmethod
-    def bring_interfaces_up(cls, ip, primary_interface, secondary_interface):
+    def bring_interfaces_up(self, ip, primary_interface, secondary_interface):
         if ip.version == 4:
-            super(RH, cls).bring_interfaces_up(
+            super(RH, self).bring_interfaces_up(
                 ip, primary_interface, secondary_interface)
         else:
             # Secondary interface is not present in IPv6 configuration
-            cls._bring_if_down(primary_interface)
-            cls._bring_if_up(primary_interface, 'VIP')
+            self._bring_if_down(primary_interface)
+            self._bring_if_up(primary_interface, 'VIP')
 
     def has_ifup_all(self):
         return False
+
+    def _write_ifup_ifdown_local_scripts_if_possible(self):
+        if self._check_ifup_ifdown_local_scripts_exists():
+            template_ifup_local = j2_env.get_template(
+                self.ETH_IFUP_LOCAL_SCRIPT)
+            self.write_port_interface_if_local_scripts(template_ifup_local)
+            template_ifdown_local = j2_env.get_template(
+                self.ETH_IFDOWN_LOCAL_SCRIPT)
+            self.write_port_interface_if_local_scripts(template_ifdown_local,
+                                                       ifup=False)
+
+    def _check_ifup_ifdown_local_scripts_exists(self):
+        file_names = ['ifup-local', 'ifdown-local']
+        target_dir = '/sbin/'
+        res = []
+        for file_name in file_names:
+            if os.path.exists(os.path.join(target_dir, file_name)):
+                res.append(True)
+            else:
+                res.append(False)
+
+        # This means we only add the scripts when both of them are non-exists
+        return not any(res)
+
+    def write_port_interface_if_local_scripts(
+            self, template_script, ifup=True):
+        file_name = 'ifup' + '-local'
+        mode = stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH
+        flags = os.O_WRONLY | os.O_CREAT | os.O_TRUNC
+        if not ifup:
+            file_name = 'ifdown' + '-local'
+        with os.fdopen(
+                os.open(os.path.join(
+                    '/sbin/', file_name), flags, mode), 'w') as text_file:
+            text = template_script.render()
+            text_file.write(text)
+        os.chmod(os.path.join('/sbin/', file_name), stat.S_IEXEC)
+
+
+class CentOS(RH):
+
+    PACKAGE_NAME_MAP = {'haproxy': 'haproxy18'}
+
+    @classmethod
+    def is_os_name(cls, os_name):
+        return os_name in ['centos']
