@@ -13,9 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import logging
-
 from oslo_config import cfg
+from oslo_log import log as logging
+import oslo_messaging as messaging
 from oslo_utils import excutils
 import pecan
 from wsme import types as wtypes
@@ -35,14 +35,13 @@ class AmphoraController(base.BaseController):
 
     def __init__(self):
         super(AmphoraController, self).__init__()
-        self.handler = self.handler.amphora
 
     @wsme_pecan.wsexpose(amp_types.AmphoraRootResponse, wtypes.text,
                          [wtypes.text], ignore_extra_args=True)
     def get_one(self, id, fields=None):
         """Gets a single amphora's details."""
         context = pecan.request.context.get('octavia_context')
-        db_amp = self._get_db_amp(context.session, id)
+        db_amp = self._get_db_amp(context.session, id, show_deleted=False)
 
         self._auth_validate_action(context, context.project_id,
                                    constants.RBAC_GET_ONE)
@@ -63,7 +62,7 @@ class AmphoraController(base.BaseController):
         self._auth_validate_action(context, context.project_id,
                                    constants.RBAC_GET_ALL)
 
-        db_amp, links = self.repositories.amphora.get_all(
+        db_amp, links = self.repositories.amphora.get_all_API_list(
             context.session, show_deleted=False,
             pagination_helper=pcontext.get(constants.PAGINATION_HELPER))
         result = self._convert_db_to_type(
@@ -80,11 +79,12 @@ class AmphoraController(base.BaseController):
         Currently it checks if this was a failover request and routes
         the request to the FailoverController.
         """
-        if amphora_id and len(remainder):
+        if amphora_id and remainder:
             controller = remainder[0]
             remainder = remainder[1:]
             if controller == 'failover':
                 return FailoverController(amp_id=amphora_id), remainder
+        return None
 
 
 class FailoverController(base.BaseController):
@@ -92,7 +92,12 @@ class FailoverController(base.BaseController):
 
     def __init__(self, amp_id):
         super(FailoverController, self).__init__()
-        self.handler = self.handler.amphora
+        topic = cfg.CONF.oslo_messaging.topic
+        self.transport = messaging.get_rpc_transport(cfg.CONF)
+        self.target = messaging.Target(
+            namespace=constants.RPC_NAMESPACE_CONTROLLER_AGENT,
+            topic=topic, version="1.0", fanout=False)
+        self.client = messaging.RPCClient(self.transport, target=self.target)
         self.amp_id = amp_id
 
     @wsme_pecan.wsexpose(None, wtypes.text, status_code=202)
@@ -100,19 +105,27 @@ class FailoverController(base.BaseController):
         """Fails over an amphora"""
         pcontext = pecan.request.context
         context = pcontext.get('octavia_context')
-        db_amp = self._get_db_amp(context.session, self.amp_id)
+        db_amp = self._get_db_amp(context.session, self.amp_id,
+                                  show_deleted=False)
 
-        self._auth_validate_action(
-            context, db_amp.load_balancer.project_id,
-            constants.RBAC_PUT_FAILOVER)
+        # Check to see if the amphora is a spare (not associated with an LB)
+        if db_amp.load_balancer:
+            self._auth_validate_action(
+                context, db_amp.load_balancer.project_id,
+                constants.RBAC_PUT_FAILOVER)
+
+            self.repositories.load_balancer.test_and_set_provisioning_status(
+                context.session, db_amp.load_balancer_id,
+                status=constants.PENDING_UPDATE, raise_exception=True)
+        else:
+            self._auth_validate_action(
+                context, context.project_id, constants.RBAC_PUT_FAILOVER)
 
-        self.repositories.load_balancer.test_and_set_provisioning_status(
-            context.session, db_amp.load_balancer_id,
-            status=constants.PENDING_UPDATE, raise_exception=True)
         try:
-            LOG.info("Sending failover request for amphora %s to the handler",
+            LOG.info("Sending failover request for amphora %s to the queue",
                      self.amp_id)
-            self.handler.failover(db_amp)
+            payload = {constants.AMPHORA_ID: db_amp.id}
+            self.client.cast({}, 'failover_amphora', **payload)
         except Exception:
             with excutils.save_and_reraise_exception(reraise=False):
                 self.repositories.load_balancer.update(
