@@ -12,14 +12,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import logging
-
 from oslo_db import exception as odb_exceptions
+from oslo_log import log as logging
 from oslo_utils import excutils
 import pecan
 from wsme import types as wtypes
 from wsmeext import pecan as wsme_pecan
 
+from octavia.api.drivers import data_models as driver_dm
+from octavia.api.drivers import driver_factory
+from octavia.api.drivers import utils as driver_utils
 from octavia.api.v2.controllers import base
 from octavia.api.v2.types import l7rule as l7rule_types
 from octavia.common import constants
@@ -39,14 +41,14 @@ class L7RuleController(base.BaseController):
     def __init__(self, l7policy_id):
         super(L7RuleController, self).__init__()
         self.l7policy_id = l7policy_id
-        self.handler = self.handler.l7rule
 
     @wsme_pecan.wsexpose(l7rule_types.L7RuleRootResponse, wtypes.text,
                          [wtypes.text], ignore_extra_args=True)
     def get(self, id, fields=None):
         """Gets a single l7rule's details."""
         context = pecan.request.context.get('octavia_context')
-        db_l7rule = self._get_db_l7rule(context.session, id)
+        db_l7rule = self._get_db_l7rule(context.session, id,
+                                        show_deleted=False)
 
         self._auth_validate_action(context, db_l7rule.project_id,
                                    constants.RBAC_GET_ONE)
@@ -64,7 +66,8 @@ class L7RuleController(base.BaseController):
         pcontext = pecan.request.context
         context = pcontext.get('octavia_context')
 
-        l7policy = self._get_db_l7policy(context.session, self.l7policy_id)
+        l7policy = self._get_db_l7policy(context.session, self.l7policy_id,
+                                         show_deleted=False)
 
         self._auth_validate_action(context, l7policy.project_id,
                                    constants.RBAC_GET_ALL)
@@ -84,10 +87,6 @@ class L7RuleController(base.BaseController):
         l7policy = self._get_db_l7policy(session, self.l7policy_id)
         listener_id = l7policy.listener_id
         load_balancer_id = l7policy.listener.load_balancer_id
-        # Check the parent is not locked for some reason (ERROR, etc.)
-        if l7policy.provisioning_status not in constants.MUTABLE_STATUSES:
-            raise exceptions.ImmutableObject(resource='L7Policy',
-                                             id=self.l7policy_id)
         if not self.repositories.test_and_set_lb_and_listeners_prov_status(
                 session, load_balancer_id,
                 constants.PENDING_UPDATE, constants.PENDING_UPDATE,
@@ -131,23 +130,6 @@ class L7RuleController(base.BaseController):
             # do not give any information as to what constraint failed
             raise exceptions.InvalidOption(value='', option='')
 
-    def _send_l7rule_to_handler(self, session, db_l7rule):
-        try:
-            LOG.info("Sending Creation of L7Rule %s to handler", db_l7rule.id)
-            self.handler.create(db_l7rule)
-        except Exception:
-            with excutils.save_and_reraise_exception(
-                    reraise=False), db_api.get_lock_session() as lock_session:
-                self._reset_lb_listener_policy_statuses(lock_session)
-                # L7Rule now goes to ERROR
-                self.repositories.l7rule.update(
-                    lock_session, db_l7rule.id,
-                    provisioning_status=constants.ERROR)
-        db_l7rule = self._get_db_l7rule(session, db_l7rule.id)
-        result = self._convert_db_to_type(db_l7rule,
-                                          l7rule_types.L7RuleResponse)
-        return l7rule_types.L7RuleRootResponse(rule=result)
-
     @wsme_pecan.wsexpose(l7rule_types.L7RuleRootResponse,
                          body=l7rule_types.L7RuleRootPOST, status_code=201)
     def post(self, rule_):
@@ -158,13 +140,22 @@ class L7RuleController(base.BaseController):
         except Exception as e:
             raise exceptions.L7RuleValidation(error=e)
         context = pecan.request.context.get('octavia_context')
-        l7rule.project_id = self._get_l7policy_project_id(context.session,
-                                                          self.l7policy_id)
+
+        db_l7policy = self._get_db_l7policy(context.session, self.l7policy_id,
+                                            show_deleted=False)
+        load_balancer_id, listener_id = self._get_listener_and_loadbalancer_id(
+            db_l7policy)
+        l7rule.project_id, provider = self._get_lb_project_id_provider(
+            context.session, load_balancer_id)
+
         self._check_l7policy_max_rules(context.session)
 
         self._auth_validate_action(context, l7rule.project_id,
                                    constants.RBAC_POST)
 
+        # Load the driver early as it also provides validation
+        driver = driver_factory.get_driver(provider)
+
         lock_session = db_api.get_session(autocommit=False)
         try:
             l7rule_dict = db_prepare.create_l7rule(
@@ -173,12 +164,26 @@ class L7RuleController(base.BaseController):
             self._test_lb_listener_policy_statuses(context.session)
 
             db_l7rule = self._validate_create_l7rule(lock_session, l7rule_dict)
+
+            # Prepare the data for the driver data model
+            provider_l7rule = (
+                driver_utils.db_l7rule_to_provider_l7rule(db_l7rule))
+
+            # Dispatch to the driver
+            LOG.info("Sending create L7 Rule %s to provider %s",
+                     db_l7rule.id, driver.name)
+            driver_utils.call_provider(
+                driver.name, driver.l7rule_create, provider_l7rule)
+
             lock_session.commit()
         except Exception:
             with excutils.save_and_reraise_exception():
                 lock_session.rollback()
 
-        return self._send_l7rule_to_handler(context.session, db_l7rule)
+        db_l7rule = self._get_db_l7rule(context.session, db_l7rule.id)
+        result = self._convert_db_to_type(db_l7rule,
+                                          l7rule_types.L7RuleResponse)
+        return l7rule_types.L7RuleRootResponse(rule=result)
 
     def _graph_create(self, lock_session, rule_dict):
         try:
@@ -197,40 +202,59 @@ class L7RuleController(base.BaseController):
         """Updates a l7rule."""
         l7rule = l7rule_.rule
         context = pecan.request.context.get('octavia_context')
-        db_l7rule = self._get_db_l7rule(context.session, id)
-
-        # Handle the invert unset
-        if l7rule.invert is None:
-            l7rule.invert = False
-
+        db_l7rule = self._get_db_l7rule(context.session, id,
+                                        show_deleted=False)
         new_l7rule = db_l7rule.to_dict()
         new_l7rule.update(l7rule.to_dict())
         new_l7rule = data_models.L7Rule.from_dict(new_l7rule)
 
-        self._auth_validate_action(context, db_l7rule.project_id,
-                                   constants.RBAC_PUT)
+        db_l7policy = self._get_db_l7policy(context.session, self.l7policy_id,
+                                            show_deleted=False)
+        load_balancer_id, listener_id = self._get_listener_and_loadbalancer_id(
+            db_l7policy)
+        project_id, provider = self._get_lb_project_id_provider(
+            context.session, load_balancer_id)
+
+        self._auth_validate_action(context, project_id, constants.RBAC_PUT)
 
         try:
             validate.l7rule_data(new_l7rule)
         except Exception as e:
             raise exceptions.L7RuleValidation(error=e)
-        self._test_lb_listener_policy_statuses(context.session)
 
-        self.repositories.l7rule.update(
-            context.session, db_l7rule.id,
-            provisioning_status=constants.PENDING_UPDATE)
+        # Load the driver early as it also provides validation
+        driver = driver_factory.get_driver(provider)
 
-        try:
-            LOG.info("Sending Update of L7Rule %s to handler", id)
-            self.handler.update(db_l7rule, l7rule)
-        except Exception:
-            with excutils.save_and_reraise_exception(
-                    reraise=False), db_api.get_lock_session() as lock_session:
-                self._reset_lb_listener_policy_statuses(lock_session)
-                # L7Rule now goes to ERROR
-                self.repositories.l7rule.update(
-                    lock_session, db_l7rule.id,
-                    provisioning_status=constants.ERROR)
+        with db_api.get_lock_session() as lock_session:
+
+            self._test_lb_listener_policy_statuses(lock_session)
+
+            # Prepare the data for the driver data model
+            l7rule_dict = l7rule.to_dict(render_unsets=False)
+            l7rule_dict['id'] = id
+            provider_l7rule_dict = (
+                driver_utils.l7rule_dict_to_provider_dict(l7rule_dict))
+
+            # Also prepare the baseline object data
+            old_provider_l7rule = driver_utils.db_l7rule_to_provider_l7rule(
+                db_l7rule)
+
+            # Dispatch to the driver
+            LOG.info("Sending update L7 Rule %s to provider %s", id,
+                     driver.name)
+            driver_utils.call_provider(
+                driver.name, driver.l7rule_update,
+                old_provider_l7rule,
+                driver_dm.L7Rule.from_dict(provider_l7rule_dict))
+
+            # Update the database to reflect what the driver just accepted
+            l7rule.provisioning_status = constants.PENDING_UPDATE
+            db_l7rule_dict = l7rule.to_dict(render_unsets=False)
+            self.repositories.l7rule.update(lock_session, id, **db_l7rule_dict)
+
+        # Force SQL alchemy to query the DB, otherwise we get inconsistent
+        # results
+        context.session.expire_all()
         db_l7rule = self._get_db_l7rule(context.session, id)
         result = self._convert_db_to_type(db_l7rule,
                                           l7rule_types.L7RuleResponse)
@@ -240,28 +264,35 @@ class L7RuleController(base.BaseController):
     def delete(self, id):
         """Deletes a l7rule."""
         context = pecan.request.context.get('octavia_context')
-        db_l7rule = self._get_db_l7rule(context.session, id)
+        db_l7rule = self._get_db_l7rule(context.session, id,
+                                        show_deleted=False)
 
-        self._auth_validate_action(context, db_l7rule.project_id,
-                                   constants.RBAC_DELETE)
+        db_l7policy = self._get_db_l7policy(context.session, self.l7policy_id,
+                                            show_deleted=False)
+        load_balancer_id, listener_id = self._get_listener_and_loadbalancer_id(
+            db_l7policy)
+        project_id, provider = self._get_lb_project_id_provider(
+            context.session, load_balancer_id)
+
+        self._auth_validate_action(context, project_id, constants.RBAC_DELETE)
 
         if db_l7rule.provisioning_status == constants.DELETED:
             return
 
-        self._test_lb_listener_policy_statuses(context.session)
+        # Load the driver early as it also provides validation
+        driver = driver_factory.get_driver(provider)
 
-        self.repositories.l7rule.update(
-            context.session, db_l7rule.id,
-            provisioning_status=constants.PENDING_DELETE)
+        with db_api.get_lock_session() as lock_session:
 
-        try:
-            LOG.info("Sending Deletion of L7Rule %s to handler", db_l7rule.id)
-            self.handler.delete(db_l7rule)
-        except Exception:
-            with excutils.save_and_reraise_exception(
-                    reraise=False), db_api.get_lock_session() as lock_session:
-                self._reset_lb_listener_policy_statuses(lock_session)
-                # L7Rule now goes to ERROR
-                self.repositories.l7rule.update(
-                    lock_session, db_l7rule.id,
-                    provisioning_status=constants.ERROR)
+            self._test_lb_listener_policy_statuses(lock_session)
+
+            self.repositories.l7rule.update(
+                lock_session, db_l7rule.id,
+                provisioning_status=constants.PENDING_DELETE)
+
+            LOG.info("Sending delete L7 Rule %s to provider %s", id,
+                     driver.name)
+            provider_l7rule = (
+                driver_utils.db_l7rule_to_provider_l7rule(db_l7rule))
+            driver_utils.call_provider(driver.name, driver.l7rule_delete,
+                                       provider_l7rule)
