@@ -13,14 +13,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import logging
-
 from oslo_db import exception as odb_exceptions
+from oslo_log import log as logging
 from oslo_utils import excutils
 import pecan
 from wsme import types as wtypes
 from wsmeext import pecan as wsme_pecan
 
+from octavia.api.drivers import data_models as driver_dm
+from octavia.api.drivers import driver_factory
+from octavia.api.drivers import utils as driver_utils
 from octavia.api.v2.controllers import base
 from octavia.api.v2.types import member as member_types
 from octavia.common import constants
@@ -40,20 +42,18 @@ class MemberController(base.BaseController):
     def __init__(self, pool_id):
         super(MemberController, self).__init__()
         self.pool_id = pool_id
-        self.handler = self.handler.member
 
     @wsme_pecan.wsexpose(member_types.MemberRootResponse, wtypes.text,
                          [wtypes.text], ignore_extra_args=True)
     def get(self, id, fields=None):
         """Gets a single pool member's details."""
         context = pecan.request.context.get('octavia_context')
-        db_member = self._get_db_member(context.session, id)
+        db_member = self._get_db_member(context.session, id,
+                                        show_deleted=False)
 
         self._auth_validate_action(context, db_member.project_id,
                                    constants.RBAC_GET_ONE)
 
-        self._validate_pool_id(id, db_member.pool_id)
-
         result = self._convert_db_to_type(
             db_member, member_types.MemberResponse)
         if fields is not None:
@@ -67,7 +67,8 @@ class MemberController(base.BaseController):
         pcontext = pecan.request.context
         context = pcontext.get('octavia_context')
 
-        pool = self._get_db_pool(context.session, self.pool_id)
+        pool = self._get_db_pool(context.session, self.pool_id,
+                                 show_deleted=False)
 
         self._auth_validate_action(context, pool.project_id,
                                    constants.RBAC_GET_ALL)
@@ -97,9 +98,6 @@ class MemberController(base.BaseController):
         # We need to verify that any listeners referencing this member's
         # pool are also mutable
         pool = self._get_db_pool(session, self.pool_id)
-        # Check the parent is not locked for some reason (ERROR, etc.)
-        if pool.provisioning_status not in constants.MUTABLE_STATUSES:
-            raise exceptions.ImmutableObject(resource='Pool', id=self.pool_id)
         load_balancer_id = pool.load_balancer_id
         if not self.repositories.test_and_set_lb_and_listeners_prov_status(
                 session, load_balancer_id,
@@ -111,21 +109,6 @@ class MemberController(base.BaseController):
             raise exceptions.ImmutableObject(resource='Load Balancer',
                                              id=load_balancer_id)
 
-    def _reset_lb_listener_pool_statuses(self, session, member=None):
-        # Setting LB + listener + pool back to active because this should be a
-        # recoverable error
-        pool = self._get_db_pool(session, self.pool_id)
-        load_balancer_id = pool.load_balancer_id
-        self.repositories.load_balancer.update(
-            session, load_balancer_id,
-            provisioning_status=constants.ACTIVE)
-        for listener in self._get_affected_listener_ids(session, member):
-            self.repositories.listener.update(
-                session, listener,
-                provisioning_status=constants.ACTIVE)
-        self.repositories.pool.update(session, self.pool_id,
-                                      provisioning_status=constants.ACTIVE)
-
     def _validate_create_member(self, lock_session, member_dict):
         """Validate creating member on pool."""
         try:
@@ -146,28 +129,6 @@ class MemberController(base.BaseController):
             # do not give any information as to what constraint failed
             raise exceptions.InvalidOption(value='', option='')
 
-    def _send_member_to_handler(self, session, db_member):
-        try:
-            LOG.info("Sending Creation of Member %s to handler", db_member.id)
-            self.handler.create(db_member)
-        except Exception:
-            with excutils.save_and_reraise_exception(
-                    reraise=False), db_api.get_lock_session() as lock_session:
-                self._reset_lb_listener_pool_statuses(
-                    lock_session, member=db_member)
-                # Member now goes to ERROR
-                self.repositories.member.update(
-                    lock_session, db_member.id,
-                    provisioning_status=constants.ERROR)
-        db_member = self._get_db_member(session, db_member.id)
-        result = self._convert_db_to_type(db_member,
-                                          member_types.MemberResponse)
-        return member_types.MemberRootResponse(member=result)
-
-    def _validate_pool_id(self, member_id, db_member_pool_id):
-        if db_member_pool_id != self.pool_id:
-            raise exceptions.NotFound(resource='Member', id=member_id)
-
     @wsme_pecan.wsexpose(member_types.MemberRootResponse,
                          body=member_types.MemberRootPOST, status_code=201)
     def post(self, member_):
@@ -182,12 +143,15 @@ class MemberController(base.BaseController):
             raise exceptions.NotFound(resource='Subnet',
                                       id=member.subnet_id)
         pool = self.repositories.pool.get(context.session, id=self.pool_id)
-        member.project_id = self._get_lb_project_id(context.session,
-                                                    pool.load_balancer_id)
+        member.project_id, provider = self._get_lb_project_id_provider(
+            context.session, pool.load_balancer_id)
 
         self._auth_validate_action(context, member.project_id,
                                    constants.RBAC_POST)
 
+        # Load the driver early as it also provides validation
+        driver = driver_factory.get_driver(provider)
+
         lock_session = db_api.get_session(autocommit=False)
         try:
             if self.repositories.check_quota_met(
@@ -204,12 +168,26 @@ class MemberController(base.BaseController):
             self._test_lb_and_listener_and_pool_statuses(lock_session)
 
             db_member = self._validate_create_member(lock_session, member_dict)
+
+            # Prepare the data for the driver data model
+            provider_member = (
+                driver_utils.db_member_to_provider_member(db_member))
+
+            # Dispatch to the driver
+            LOG.info("Sending create Member %s to provider %s",
+                     db_member.id, driver.name)
+            driver_utils.call_provider(
+                driver.name, driver.member_create, provider_member)
+
             lock_session.commit()
         except Exception:
             with excutils.save_and_reraise_exception():
                 lock_session.rollback()
 
-        return self._send_member_to_handler(context.session, db_member)
+        db_member = self._get_db_member(context.session, db_member.id)
+        result = self._convert_db_to_type(db_member,
+                                          member_types.MemberResponse)
+        return member_types.MemberRootResponse(member=result)
 
     def _graph_create(self, lock_session, member_dict):
         pool = self.repositories.pool.get(lock_session, id=self.pool_id)
@@ -226,31 +204,49 @@ class MemberController(base.BaseController):
         """Updates a pool member."""
         member = member_.member
         context = pecan.request.context.get('octavia_context')
-        db_member = self._get_db_member(context.session, id)
+        db_member = self._get_db_member(context.session, id,
+                                        show_deleted=False)
 
-        self._auth_validate_action(context, db_member.project_id,
-                                   constants.RBAC_PUT)
+        pool = self.repositories.pool.get(context.session,
+                                          id=db_member.pool_id)
+        project_id, provider = self._get_lb_project_id_provider(
+            context.session, pool.load_balancer_id)
 
-        self._test_lb_and_listener_and_pool_statuses(context.session,
-                                                     member=db_member)
-        self.repositories.member.update(
-            context.session, db_member.id,
-            provisioning_status=constants.PENDING_UPDATE)
+        self._auth_validate_action(context, project_id, constants.RBAC_PUT)
 
-        self._validate_pool_id(id, db_member.pool_id)
+        # Load the driver early as it also provides validation
+        driver = driver_factory.get_driver(provider)
 
-        try:
-            LOG.info("Sending Update of Member %s to handler", id)
-            self.handler.update(db_member, member)
-        except Exception:
-            with excutils.save_and_reraise_exception(
-                    reraise=False), db_api.get_lock_session() as lock_session:
-                self._reset_lb_listener_pool_statuses(
-                    lock_session, member=db_member)
-                # Member now goes to ERROR
-                self.repositories.member.update(
-                    lock_session, db_member.id,
-                    provisioning_status=constants.ERROR)
+        with db_api.get_lock_session() as lock_session:
+            self._test_lb_and_listener_and_pool_statuses(lock_session,
+                                                         member=db_member)
+
+            # Prepare the data for the driver data model
+            member_dict = member.to_dict(render_unsets=False)
+            member_dict['id'] = id
+            provider_member_dict = (
+                driver_utils.member_dict_to_provider_dict(member_dict))
+
+            # Also prepare the baseline object data
+            old_provider_member = driver_utils.db_member_to_provider_member(
+                db_member)
+
+            # Dispatch to the driver
+            LOG.info("Sending update Member %s to provider %s", id,
+                     driver.name)
+            driver_utils.call_provider(
+                driver.name, driver.member_update,
+                old_provider_member,
+                driver_dm.Member.from_dict(provider_member_dict))
+
+            # Update the database to reflect what the driver just accepted
+            member.provisioning_status = constants.PENDING_UPDATE
+            db_member_dict = member.to_dict(render_unsets=False)
+            self.repositories.member.update(lock_session, id, **db_member_dict)
+
+        # Force SQL alchemy to query the DB, otherwise we get inconsistent
+        # results
+        context.session.expire_all()
         db_member = self._get_db_member(context.session, id)
         result = self._convert_db_to_type(db_member,
                                           member_types.MemberResponse)
@@ -260,34 +256,32 @@ class MemberController(base.BaseController):
     def delete(self, id):
         """Deletes a pool member."""
         context = pecan.request.context.get('octavia_context')
-        db_member = self._get_db_member(context.session, id)
+        db_member = self._get_db_member(context.session, id,
+                                        show_deleted=False)
 
-        self._auth_validate_action(context, db_member.project_id,
-                                   constants.RBAC_DELETE)
+        pool = self.repositories.pool.get(context.session,
+                                          id=db_member.pool_id)
+        project_id, provider = self._get_lb_project_id_provider(
+            context.session, pool.load_balancer_id)
 
-        if db_member.provisioning_status == constants.DELETED:
-            return
+        self._auth_validate_action(context, project_id, constants.RBAC_DELETE)
 
-        self._validate_pool_id(id, db_member.pool_id)
+        # Load the driver early as it also provides validation
+        driver = driver_factory.get_driver(provider)
 
-        self._test_lb_and_listener_and_pool_statuses(context.session,
-                                                     member=db_member)
-        self.repositories.member.update(
-            context.session, db_member.id,
-            provisioning_status=constants.PENDING_DELETE)
+        with db_api.get_lock_session() as lock_session:
+            self._test_lb_and_listener_and_pool_statuses(lock_session,
+                                                         member=db_member)
+            self.repositories.member.update(
+                lock_session, db_member.id,
+                provisioning_status=constants.PENDING_DELETE)
 
-        try:
-            LOG.info("Sending Deletion of Member %s to handler", db_member.id)
-            self.handler.delete(db_member)
-        except Exception:
-            with excutils.save_and_reraise_exception(
-                    reraise=False), db_api.get_lock_session() as lock_session:
-                self._reset_lb_listener_pool_statuses(
-                    lock_session, member=db_member)
-                # Member now goes to ERROR
-                self.repositories.member.update(
-                    lock_session, db_member.id,
-                    provisioning_status=constants.ERROR)
+            LOG.info("Sending delete Member %s to provider %s", id,
+                     driver.name)
+            provider_member = (
+                driver_utils.db_member_to_provider_member(db_member))
+            driver_utils.call_provider(driver.name, driver.member_delete,
+                                       provider_member)
 
 
 class MembersController(MemberController):
@@ -305,13 +299,23 @@ class MembersController(MemberController):
         db_pool = self._get_db_pool(context.session, self.pool_id)
         old_members = db_pool.members
 
+        project_id, provider = self._get_lb_project_id_provider(
+            context.session, db_pool.load_balancer_id)
+
         # Check POST+PUT+DELETE since this operation is all of 'CUD'
-        self._auth_validate_action(context, db_pool.project_id,
-                                   constants.RBAC_POST)
-        self._auth_validate_action(context, db_pool.project_id,
-                                   constants.RBAC_PUT)
-        self._auth_validate_action(context, db_pool.project_id,
-                                   constants.RBAC_DELETE)
+        self._auth_validate_action(context, project_id, constants.RBAC_POST)
+        self._auth_validate_action(context, project_id, constants.RBAC_PUT)
+        self._auth_validate_action(context, project_id, constants.RBAC_DELETE)
+
+        # Validate member subnets
+        for member in members:
+            if member.subnet_id and not validate.subnet_exists(
+                    member.subnet_id):
+                raise exceptions.NotFound(resource='Subnet',
+                                          id=member.subnet_id)
+
+        # Load the driver early as it also provides validation
+        driver = driver_factory.get_driver(provider)
 
         with db_api.get_lock_session() as lock_session:
             self._test_lb_and_listener_and_pool_statuses(lock_session)
@@ -345,25 +349,34 @@ class MembersController(MemberController):
                 if (m.ip_address, m.protocol_port) not in new_member_uniques:
                     deleted_members.append(m)
 
+            provider_members = []
             # Create new members
-            new_members_created = []
             for m in new_members:
                 m = m.to_dict(render_unsets=False)
                 m['project_id'] = db_pool.project_id
-                new_members_created.append(self._graph_create(lock_session, m))
+                created_member = self._graph_create(lock_session, m)
+                provider_member = driver_utils.db_member_to_provider_member(
+                    created_member)
+                provider_members.append(provider_member)
             # Update old members
             for m in updated_members:
+                m.provisioning_status = constants.PENDING_UPDATE
+                db_member_dict = m.to_dict(render_unsets=False)
+                db_member_dict.pop('id')
                 self.repositories.member.update(
-                    lock_session, m.id,
-                    provisioning_status=constants.PENDING_UPDATE)
+                    lock_session, m.id, **db_member_dict)
+
+                m.pool_id = self.pool_id
+                provider_members.append(
+                    driver_utils.db_member_to_provider_member(m))
             # Delete old members
             for m in deleted_members:
                 self.repositories.member.update(
                     lock_session, m.id,
                     provisioning_status=constants.PENDING_DELETE)
 
-            LOG.info("Sending Full Member Update to handler")
-            new_member_ids = [m.id for m in new_members_created]
-            old_member_ids = [m.id for m in deleted_members]
-            self.handler.batch_update(
-                old_member_ids, new_member_ids, updated_members)
+            # Dispatch to the driver
+            LOG.info("Sending Pool %s batch member update to provider %s",
+                     db_pool.id, driver.name)
+            driver_utils.call_provider(
+                driver.name, driver.member_batch_update, provider_members)
