@@ -25,7 +25,6 @@ from oslo_db import options as db_options
 from oslo_log import log as logging
 import oslo_messaging as messaging
 
-from octavia.certificates.common import local
 from octavia.common import constants
 from octavia.common import utils
 from octavia.i18n import _
@@ -101,6 +100,25 @@ api_opts = [
                 help=_("Expose the v2 API?")),
     cfg.BoolOpt('allow_tls_terminated_listeners', default=True,
                 help=_("Allow users to create TLS Terminated listeners?")),
+    cfg.BoolOpt('allow_ping_health_monitors', default=True,
+                help=_("Allow users to create PING type Health Monitors?")),
+    cfg.DictOpt('enabled_provider_drivers',
+                help=_('List of enabled provider drivers and description '
+                       'dictionaries. Must match the driver name in the '
+                       'octavia.api.drivers entrypoint. Example: '
+                       '{\'amphora\': \'The Octavia Amphora driver.\', '
+                       '\'octavia\': \'Deprecated alias of the Octavia '
+                       'Amphora driver.\'}'),
+                default={'amphora': 'The Octavia Amphora driver.',
+                         'octavia': 'Deprecated alias of the Octavia Amphora '
+                         'driver.'}),
+    cfg.StrOpt('default_provider_driver', default='amphora',
+               help=_('Default provider driver.')),
+    cfg.IntOpt('udp_connect_min_interval_health_monitor',
+               default=3,
+               help=_("The minimum health monitor delay interval for the "
+                      "UDP-CONNECT Health Monitor type. A negative integer "
+                      "value means 'no limit'.")),
 ]
 
 # Options only used by the amphora agent
@@ -122,6 +140,9 @@ amphora_agent_opts = [
                       "controller to run before terminating the socket.")),
     # Do not specify in octavia.conf, loaded at runtime
     cfg.StrOpt('amphora_id', help=_("The amphora ID.")),
+    cfg.StrOpt('amphora_udp_driver',
+               default='keepalived_lvs',
+               help='The UDP API backend for amphora agent.'),
 ]
 
 networking_opts = [
@@ -230,6 +251,8 @@ haproxy_amphora_opts = [
                default='/var/lib/octavia/certs',
                help=_('Base directory for cert storage.')),
     cfg.StrOpt('haproxy_template', help=_('Custom haproxy template.')),
+    cfg.BoolOpt('connection_logging', default=True,
+                help=_('Set this to False to disable connection logging.')),
     cfg.IntOpt('connection_max_retries',
                default=300,
                help=_('Retry threshold for connecting to amphorae.')),
@@ -256,11 +279,6 @@ haproxy_amphora_opts = [
                default=5,
                help=_('Retry timeout between build attempts in '
                       'seconds.')),
-    cfg.StrOpt('user_group', deprecated_for_removal=True,
-               deprecated_reason='This is now automatically discovered '
-                                 ' and configured.',
-               help=_('The user group for haproxy to run under inside the '
-                      'amphora.')),
     cfg.StrOpt('haproxy_stick_size', default='10k',
                help=_('Size of the HAProxy stick table. Accepts k, m, g '
                       'suffixes.  Example: 10k')),
@@ -591,9 +609,6 @@ cfg.CONF.register_opts(neutron_opts, group='neutron')
 cfg.CONF.register_opts(quota_opts, group='quotas')
 
 
-cfg.CONF.register_opts(local.certgen_opts, group='certificates')
-cfg.CONF.register_opts(local.certmgr_opts, group='certificates')
-
 # Ensure that the control exchange is set correctly
 messaging.set_transport_defaults(control_exchange='octavia')
 _SQL_CONNECTION_DEFAULT = 'sqlite://'
