@@ -38,7 +38,7 @@ class OctaviaException(Exception):
 
     def __init__(self, *args, **kwargs):
         try:
-            if len(args) > 0:
+            if args:
                 self.message = args[0]
                 self.orig_msg = kwargs.get('orig_msg')
                 self.orig_code = kwargs.get('orig_code')
@@ -103,7 +103,7 @@ class L7RuleValidation(APIException):
 
 
 class SingleCreateDetailsMissing(APIException):
-    msg = _("Missing details for %(type)s object: %(name)s")
+    msg = _("Missing details for %(type)s object: %(name)")
     code = 400
 
 
@@ -224,10 +224,6 @@ class GlanceNoTaggedImages(OctaviaException):
     message = _("No Glance images are tagged with %(tag)s tag.")
 
 
-class NoSuitableAmphoraException(OctaviaException):
-    message = _('Unable to allocate an amphora due to: %(msg)s')
-
-
 # This is an internal use exception for the taskflow work flow
 # and will not be exposed to the customer.  This means it is a
 # normal part of operation while waiting for compute to go active
@@ -340,3 +336,30 @@ class InvalidLimit(APIException):
 
 class MissingVIPSecurityGroup(OctaviaException):
     message = _('VIP security group is missing for load balancer: %(lb_id)s')
+
+
+class ProviderNotEnabled(APIException):
+    msg = _("Provider '%(prov)s' is not enabled.")
+    code = 400
+
+
+class ProviderNotFound(APIException):
+    msg = _("Provider '%(prov)s' was not found.")
+    code = 501
+
+
+class ProviderDriverError(APIException):
+    msg = _("Provider '%(prov)s' reports error: %(user_msg)s")
+    code = 500
+
+
+class ProviderNotImplementedError(APIException):
+    msg = _("Provider '%(prov)s' does not support a requested action: "
+            "%(user_msg)s")
+    code = 501
+
+
+class ProviderUnsupportedOptionError(APIException):
+    msg = _("Provider '%(prov)s' does not support a requested option: "
+            "%(user_msg)s")
+    code = 501
