@@ -176,10 +176,28 @@ class UpdateHealthDb(update_base.HealthUpdateBase):
                 LOG.debug('Received a health heartbeat from amphora {0} with '
                           'IP {1} that should not exist. This amphora may be '
                           'in the process of being deleted, in which case you '
-                          'will only see this message a few times. However if '
-                          'it is repeating this amphora should be manually '
-                          'deleted.'.format(health['id'], srcaddr))
-                return
+                          'will only see this message a few '
+                          'times'.format(health['id'], srcaddr))
+                if not amp:
+                    LOG.warning('The amphora {0} with IP {1} is missing from '
+                                'the DB, so it cannot be automatically '
+                                'deleted (the compute_id is unknown). An '
+                                'operator must manually delete it from the '
+                                'compute service.'.format(health['id'],
+                                                          srcaddr))
+                    return
+                # delete the amp right there
+                try:
+                    compute = stevedore_driver.DriverManager(
+                        namespace='octavia.compute.drivers',
+                        name=CONF.controller_worker.compute_driver,
+                        invoke_on_load=True
+                    ).driver
+                    compute.delete(amp.compute_id)
+                    return
+                except Exception as e:
+                    LOG.info("Error deleting amp {0} with IP {1}".format(
+                        health['id'], srcaddr), e)
             expected_listener_count = 0
 
         listeners = health['listeners']
