@@ -13,7 +13,13 @@
 # under the License.
 #
 
-import logging
+
+from oslo_config import cfg
+from oslo_log import log as logging
+from oslo_utils import excutils
+from sqlalchemy.orm import exc as db_exceptions
+from taskflow.listeners import logging as tf_logging
+import tenacity
 
 from octavia.common import base_taskflow
 from octavia.common import constants
@@ -28,13 +34,18 @@ from octavia.controller.worker.flows import pool_flows
 from octavia.db import api as db_apis
 from octavia.db import repositories as repo
 
-from oslo_config import cfg
-from oslo_utils import excutils
-from taskflow.listeners import logging as tf_logging
-
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
+RETRY_ATTEMPTS = 15
+RETRY_INITIAL_DELAY = 1
+RETRY_BACKOFF = 1
+RETRY_MAX = 5
+
+
+def _is_provisioning_status_pending_update(lb_obj):
+    return not lb_obj.provisioning_status == constants.PENDING_UPDATE
+
 
 class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
 
@@ -59,8 +70,36 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         self._l7policy_repo = repo.L7PolicyRepository()
         self._l7rule_repo = repo.L7RuleRepository()
 
+        self._exclude_result_logging_tasks = (
+            constants.ROLE_STANDALONE + '-' +
+            constants.CREATE_AMP_FOR_LB_SUBFLOW + '-' +
+            constants.GENERATE_SERVER_PEM,
+            constants.ROLE_BACKUP + '-' +
+            constants.CREATE_AMP_FOR_LB_SUBFLOW + '-' +
+            constants.GENERATE_SERVER_PEM,
+            constants.ROLE_MASTER + '-' +
+            constants.CREATE_AMP_FOR_LB_SUBFLOW + '-' +
+            constants.GENERATE_SERVER_PEM,
+            constants.GENERATE_SERVER_PEM_TASK,
+            constants.FAILOVER_AMPHORA_FLOW + '-' +
+            constants.CREATE_AMP_FOR_LB_SUBFLOW + '-' +
+            constants.GENERATE_SERVER_PEM,
+            constants.CREATE_AMP_FOR_LB_SUBFLOW + '-' +
+            constants.UPDATE_CERT_EXPIRATION)
+
         super(ControllerWorker, self).__init__()
 
+    @tenacity.retry(
+        retry=(
+            tenacity.retry_if_result(_is_provisioning_status_pending_update) |
+            tenacity.retry_if_exception_type()),
+        wait=tenacity.wait_incrementing(
+            RETRY_INITIAL_DELAY, RETRY_BACKOFF, RETRY_MAX),
+        stop=tenacity.stop_after_attempt(RETRY_ATTEMPTS))
+    def _get_db_obj_until_pending_update(self, repo, id):
+
+        return repo.get(db_apis.get_session(), id=id)
+
     def create_amphora(self):
         """Creates an Amphora.
 
@@ -73,7 +112,9 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
             store={constants.BUILD_TYPE_PRIORITY:
                    constants.LB_CREATE_SPARES_POOL_PRIORITY}
         )
-        with tf_logging.DynamicLoggingListener(create_amp_tf, log=LOG):
+        with tf_logging.DynamicLoggingListener(
+                create_amp_tf, log=LOG,
+                hide_inputs_outputs_of=self._exclude_result_logging_tasks):
 
             create_amp_tf.run()
 
@@ -95,15 +136,24 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                                                log=LOG):
             delete_amp_tf.run()
 
+    @tenacity.retry(
+        retry=tenacity.retry_if_exception_type(db_exceptions.NoResultFound),
+        wait=tenacity.wait_incrementing(
+            RETRY_INITIAL_DELAY, RETRY_BACKOFF, RETRY_MAX),
+        stop=tenacity.stop_after_attempt(RETRY_ATTEMPTS))
     def create_health_monitor(self, health_monitor_id):
         """Creates a health monitor.
 
         :param pool_id: ID of the pool to create a health monitor on
         :returns: None
-        :raises NoSuitablePool: Unable to find the node pool
+        :raises NoResultFound: Unable to find the object
         """
         health_mon = self._health_mon_repo.get(db_apis.get_session(),
                                                id=health_monitor_id)
+        if not health_mon:
+            LOG.warning('Failed to fetch %s %s from DB. Retrying for up to '
+                        '60 seconds.', 'health_monitor', health_monitor_id)
+            raise db_exceptions.NoResultFound
 
         pool = health_mon.pool
         listeners = pool.listeners
@@ -152,8 +202,17 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         :returns: None
         :raises HMNotFound: The referenced health monitor was not found
         """
-        health_mon = self._health_mon_repo.get(db_apis.get_session(),
-                                               id=health_monitor_id)
+        health_mon = None
+        try:
+            health_mon = self._get_db_obj_until_pending_update(
+                self._health_mon_repo, health_monitor_id)
+        except tenacity.RetryError as e:
+            LOG.warning('Health monitor did not go into %s in 60 seconds. '
+                        'This either due to an in-progress Octavia upgrade '
+                        'or an overloaded and failing database. Assuming '
+                        'an upgrade is in progress and continuing.',
+                        constants.PENDING_UPDATE)
+            health_mon = e.last_attempt.result()
 
         pool = health_mon.pool
         listeners = pool.listeners
@@ -171,15 +230,25 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                                                log=LOG):
             update_hm_tf.run()
 
+    @tenacity.retry(
+        retry=tenacity.retry_if_exception_type(db_exceptions.NoResultFound),
+        wait=tenacity.wait_incrementing(
+            RETRY_INITIAL_DELAY, RETRY_BACKOFF, RETRY_MAX),
+        stop=tenacity.stop_after_attempt(RETRY_ATTEMPTS))
     def create_listener(self, listener_id):
         """Creates a listener.
 
         :param listener_id: ID of the listener to create
         :returns: None
-        :raises NoSuitableLB: Unable to find the load balancer
+        :raises NoResultFound: Unable to find the object
         """
         listener = self._listener_repo.get(db_apis.get_session(),
                                            id=listener_id)
+        if not listener:
+            LOG.warning('Failed to fetch %s %s from DB. Retrying for up to '
+                        '60 seconds.', 'listener', listener_id)
+            raise db_exceptions.NoResultFound
+
         load_balancer = listener.load_balancer
 
         create_listener_tf = self._taskflow_load(self._listener_flows.
@@ -219,8 +288,17 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         :returns: None
         :raises ListenerNotFound: The referenced listener was not found
         """
-        listener = self._listener_repo.get(db_apis.get_session(),
-                                           id=listener_id)
+        listener = None
+        try:
+            listener = self._get_db_obj_until_pending_update(
+                self._listener_repo, listener_id)
+        except tenacity.RetryError as e:
+            LOG.warning('Listener did not go into %s in 60 seconds. '
+                        'This either due to an in-progress Octavia upgrade '
+                        'or an overloaded and failing database. Assuming '
+                        'an upgrade is in progress and continuing.',
+                        constants.PENDING_UPDATE)
+            listener = e.last_attempt.result()
 
         load_balancer = listener.load_balancer
 
@@ -237,6 +315,11 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         with tf_logging.DynamicLoggingListener(update_listener_tf, log=LOG):
             update_listener_tf.run()
 
+    @tenacity.retry(
+        retry=tenacity.retry_if_exception_type(db_exceptions.NoResultFound),
+        wait=tenacity.wait_incrementing(
+            RETRY_INITIAL_DELAY, RETRY_BACKOFF, RETRY_MAX),
+        stop=tenacity.stop_after_attempt(RETRY_ATTEMPTS))
     def create_load_balancer(self, load_balancer_id):
         """Creates a load balancer by allocating Amphorae.
 
@@ -246,8 +329,13 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
 
         :param load_balancer_id: ID of the load balancer to create
         :returns: None
-        :raises NoSuitableAmphoraException: Unable to allocate an Amphora.
+        :raises NoResultFound: Unable to find the object
         """
+        lb = self._lb_repo.get(db_apis.get_session(), id=load_balancer_id)
+        if not lb:
+            LOG.warning('Failed to fetch %s %s from DB. Retrying for up to '
+                        '60 seconds.', 'load_balancer', load_balancer_id)
+            raise db_exceptions.NoResultFound
 
         store = {constants.LOADBALANCER_ID: load_balancer_id,
                  constants.BUILD_TYPE_PRIORITY:
@@ -259,12 +347,13 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
             constants.LOADBALANCER_TOPOLOGY: topology
         }
 
-        lb = self._lb_repo.get(db_apis.get_session(), id=load_balancer_id)
         create_lb_flow = self._lb_flows.get_create_load_balancer_flow(
             topology=topology, listeners=lb.listeners)
 
         create_lb_tf = self._taskflow_load(create_lb_flow, store=store)
-        with tf_logging.DynamicLoggingListener(create_lb_tf, log=LOG):
+        with tf_logging.DynamicLoggingListener(
+                create_lb_tf, log=LOG,
+                hide_inputs_outputs_of=self._exclude_result_logging_tasks):
             create_lb_tf.run()
 
     def delete_load_balancer(self, load_balancer_id, cascade=False):
@@ -298,8 +387,18 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         :returns: None
         :raises LBNotFound: The referenced load balancer was not found
         """
-        lb = self._lb_repo.get(db_apis.get_session(),
-                               id=load_balancer_id)
+        lb = None
+        try:
+            lb = self._get_db_obj_until_pending_update(
+                self._lb_repo, load_balancer_id)
+        except tenacity.RetryError as e:
+            LOG.warning('Load balancer did not go into %s in 60 seconds. '
+                        'This either due to an in-progress Octavia upgrade '
+                        'or an overloaded and failing database. Assuming '
+                        'an upgrade is in progress and continuing.',
+                        constants.PENDING_UPDATE)
+            lb = e.last_attempt.result()
+
         listeners, _ = self._listener_repo.get_all(
             db_apis.get_session(),
             load_balancer_id=load_balancer_id)
@@ -314,6 +413,11 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                                                log=LOG):
             update_lb_tf.run()
 
+    @tenacity.retry(
+        retry=tenacity.retry_if_exception_type(db_exceptions.NoResultFound),
+        wait=tenacity.wait_incrementing(
+            RETRY_INITIAL_DELAY, RETRY_BACKOFF, RETRY_MAX),
+        stop=tenacity.stop_after_attempt(RETRY_ATTEMPTS))
     def create_member(self, member_id):
         """Creates a pool member.
 
@@ -323,6 +427,11 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         """
         member = self._member_repo.get(db_apis.get_session(),
                                        id=member_id)
+        if not member:
+            LOG.warning('Failed to fetch %s %s from DB. Retrying for up to '
+                        '60 seconds.', 'member', member_id)
+            raise db_exceptions.NoResultFound
+
         pool = member.pool
         listeners = pool.listeners
         load_balancer = pool.load_balancer
@@ -397,8 +506,18 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         :returns: None
         :raises MemberNotFound: The referenced member was not found
         """
-        member = self._member_repo.get(db_apis.get_session(),
-                                       id=member_id)
+        member = None
+        try:
+            member = self._get_db_obj_until_pending_update(
+                self._member_repo, member_id)
+        except tenacity.RetryError as e:
+            LOG.warning('Member did not go into %s in 60 seconds. '
+                        'This either due to an in-progress Octavia upgrade '
+                        'or an overloaded and failing database. Assuming '
+                        'an upgrade is in progress and continuing.',
+                        constants.PENDING_UPDATE)
+            member = e.last_attempt.result()
+
         pool = member.pool
         listeners = pool.listeners
         load_balancer = pool.load_balancer
@@ -418,15 +537,24 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                                                log=LOG):
             update_member_tf.run()
 
+    @tenacity.retry(
+        retry=tenacity.retry_if_exception_type(db_exceptions.NoResultFound),
+        wait=tenacity.wait_incrementing(
+            RETRY_INITIAL_DELAY, RETRY_BACKOFF, RETRY_MAX),
+        stop=tenacity.stop_after_attempt(RETRY_ATTEMPTS))
     def create_pool(self, pool_id):
         """Creates a node pool.
 
         :param pool_id: ID of the pool to create
         :returns: None
-        :raises NoSuitableLB: Unable to find the load balancer
+        :raises NoResultFound: Unable to find the object
         """
         pool = self._pool_repo.get(db_apis.get_session(),
                                    id=pool_id)
+        if not pool:
+            LOG.warning('Failed to fetch %s %s from DB. Retrying for up to '
+                        '60 seconds.', 'pool', pool_id)
+            raise db_exceptions.NoResultFound
 
         listeners = pool.listeners
         load_balancer = pool.load_balancer
@@ -471,8 +599,17 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         :returns: None
         :raises PoolNotFound: The referenced pool was not found
         """
-        pool = self._pool_repo.get(db_apis.get_session(),
-                                   id=pool_id)
+        pool = None
+        try:
+            pool = self._get_db_obj_until_pending_update(
+                self._pool_repo, pool_id)
+        except tenacity.RetryError as e:
+            LOG.warning('Pool did not go into %s in 60 seconds. '
+                        'This either due to an in-progress Octavia upgrade '
+                        'or an overloaded and failing database. Assuming '
+                        'an upgrade is in progress and continuing.',
+                        constants.PENDING_UPDATE)
+            pool = e.last_attempt.result()
 
         listeners = pool.listeners
         load_balancer = pool.load_balancer
@@ -490,15 +627,24 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                                                log=LOG):
             update_pool_tf.run()
 
+    @tenacity.retry(
+        retry=tenacity.retry_if_exception_type(db_exceptions.NoResultFound),
+        wait=tenacity.wait_incrementing(
+            RETRY_INITIAL_DELAY, RETRY_BACKOFF, RETRY_MAX),
+        stop=tenacity.stop_after_attempt(RETRY_ATTEMPTS))
     def create_l7policy(self, l7policy_id):
         """Creates an L7 Policy.
 
         :param l7policy_id: ID of the l7policy to create
         :returns: None
-        :raises NoSuitableLB: Unable to find the load balancer
+        :raises NoResultFound: Unable to find the object
         """
         l7policy = self._l7policy_repo.get(db_apis.get_session(),
                                            id=l7policy_id)
+        if not l7policy:
+            LOG.warning('Failed to fetch %s %s from DB. Retrying for up to '
+                        '60 seconds.', 'l7policy', l7policy_id)
+            raise db_exceptions.NoResultFound
 
         listeners = [l7policy.listener]
         load_balancer = l7policy.listener.load_balancer
@@ -542,8 +688,17 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         :returns: None
         :raises L7PolicyNotFound: The referenced l7policy was not found
         """
-        l7policy = self._l7policy_repo.get(db_apis.get_session(),
-                                           id=l7policy_id)
+        l7policy = None
+        try:
+            l7policy = self._get_db_obj_until_pending_update(
+                self._l7policy_repo, l7policy_id)
+        except tenacity.RetryError as e:
+            LOG.warning('L7 policy did not go into %s in 60 seconds. '
+                        'This either due to an in-progress Octavia upgrade '
+                        'or an overloaded and failing database. Assuming '
+                        'an upgrade is in progress and continuing.',
+                        constants.PENDING_UPDATE)
+            l7policy = e.last_attempt.result()
 
         listeners = [l7policy.listener]
         load_balancer = l7policy.listener.load_balancer
@@ -558,15 +713,25 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                                                log=LOG):
             update_l7policy_tf.run()
 
+    @tenacity.retry(
+        retry=tenacity.retry_if_exception_type(db_exceptions.NoResultFound),
+        wait=tenacity.wait_incrementing(
+            RETRY_INITIAL_DELAY, RETRY_BACKOFF, RETRY_MAX),
+        stop=tenacity.stop_after_attempt(RETRY_ATTEMPTS))
     def create_l7rule(self, l7rule_id):
         """Creates an L7 Rule.
 
         :param l7rule_id: ID of the l7rule to create
         :returns: None
-        :raises NoSuitableLB: Unable to find the load balancer
+        :raises NoResultFound: Unable to find the object
         """
         l7rule = self._l7rule_repo.get(db_apis.get_session(),
                                        id=l7rule_id)
+        if not l7rule:
+            LOG.warning('Failed to fetch %s %s from DB. Retrying for up to '
+                        '60 seconds.', 'l7rule', l7rule_id)
+            raise db_exceptions.NoResultFound
+
         l7policy = l7rule.l7policy
         listeners = [l7policy.listener]
         load_balancer = l7policy.listener.load_balancer
@@ -612,8 +777,18 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         :returns: None
         :raises L7RuleNotFound: The referenced l7rule was not found
         """
-        l7rule = self._l7rule_repo.get(db_apis.get_session(),
-                                       id=l7rule_id)
+        l7rule = None
+        try:
+            l7rule = self._get_db_obj_until_pending_update(
+                self._l7rule_repo, l7rule_id)
+        except tenacity.RetryError as e:
+            LOG.warning('L7 rule did not go into %s in 60 seconds. '
+                        'This either due to an in-progress Octavia upgrade '
+                        'or an overloaded and failing database. Assuming '
+                        'an upgrade is in progress and continuing.',
+                        constants.PENDING_UPDATE)
+            l7rule = e.last_attempt.result()
+
         l7policy = l7rule.l7policy
         listeners = [l7policy.listener]
         load_balancer = l7policy.listener.load_balancer
@@ -641,29 +816,13 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                          constants.LOADBALANCER_ID: amp.load_balancer_id,
                          constants.BUILD_TYPE_PRIORITY: priority, }
 
-        if amp.role in (constants.ROLE_MASTER, constants.ROLE_BACKUP):
-            amp_role = 'master_or_backup'
-        elif amp.role == constants.ROLE_STANDALONE:
-            amp_role = 'standalone'
-        elif amp.role is None:
-            amp_role = 'spare'
-        else:
-            amp_role = 'undefined'
-
-        LOG.info("Perform failover for an amphora: %s",
-                 {"id": amp.id,
-                  "load_balancer_id": amp.load_balancer_id,
-                  "lb_network_ip": amp.lb_network_ip,
-                  "compute_id": amp.compute_id,
-                  "role": amp_role})
-
         if amp.status == constants.DELETED:
             LOG.warning('Amphora %s is marked DELETED in the database but '
-                        'was submitted for failover. Marking it busy in the '
+                        'was submitted for failover. Deleting it from the '
                         'amphora health table to exclude it from health '
                         'checks and skipping the failover.', amp.id)
-            self._amphora_health_repo.update(db_apis.get_session(), amp.id,
-                                             busy=True)
+            self._amphora_health_repo.delete(db_apis.get_session(),
+                                             amphora_id=amp.id)
             return
 
         if (CONF.house_keeping.spare_amphora_pool_size == 0) and (
@@ -684,15 +843,11 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                 role=amp.role, load_balancer=lb),
             store=stored_params)
 
-        with tf_logging.DynamicLoggingListener(failover_amphora_tf, log=LOG):
-            failover_amphora_tf.run()
+        with tf_logging.DynamicLoggingListener(
+                failover_amphora_tf, log=LOG,
+                hide_inputs_outputs_of=self._exclude_result_logging_tasks):
 
-        LOG.info("Successfully completed the failover for an amphora: %s",
-                 {"id": amp.id,
-                  "load_balancer_id": amp.load_balancer_id,
-                  "lb_network_ip": amp.lb_network_ip,
-                  "compute_id": amp.compute_id,
-                  "role": amp_role})
+            failover_amphora_tf.run()
 
     def failover_amphora(self, amphora_id):
         """Perform failover operations for an amphora.
@@ -717,15 +872,8 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                     db_apis.get_session(), amp.load_balancer_id,
                     provisioning_status=constants.ACTIVE)
         except Exception as e:
-            try:
-                self._lb_repo.update(
-                    db_apis.get_session(), amp.load_balancer_id,
-                    provisioning_status=constants.ERROR)
-            except Exception:
-                LOG.error("Unable to revert LB status to ERROR.")
             with excutils.save_and_reraise_exception():
-                LOG.error("Amphora %(id)s failover exception: %(exc)s",
-                          {'id': amphora_id, 'exc': e})
+                LOG.error("Failover exception: %s", e)
 
     def failover_loadbalancer(self, load_balancer_id):
         """Perform failover operations for a load balancer.
