@@ -13,10 +13,10 @@
 # under the License.
 #
 
-import logging
 import time
 
 from oslo_config import cfg
+from oslo_log import log as logging
 from stevedore import driver as stevedore_driver
 from taskflow import task
 from taskflow.types import failure
@@ -140,8 +140,7 @@ class CertComputeCreate(ComputeCreate):
         return super(CertComputeCreate, self).execute(
             amphora_id, config_drive_files=config_drive_files,
             build_type_priority=build_type_priority,
-            server_group_id=server_group_id, ports=ports,
-            )
+            server_group_id=server_group_id, ports=ports)
 
 
 class DeleteAmphoraeOnLoadBalancer(BaseComputeTask):
