@@ -34,10 +34,11 @@ class AgentJinjaTestCase(base.TestCase):
                          agent_server_cert='/etc/octavia/certs/server.pem')
         self.conf.config(group="amphora_agent",
                          agent_server_network_dir='/etc/network/interfaces.d/')
+        self.conf.config(group='amphora_agent',
+                         amphora_udp_driver='keepalived_lvs'),
         self.conf.config(group="haproxy_amphora",
                          base_cert_dir='/var/lib/octavia/certs')
         self.conf.config(group="haproxy_amphora", use_upstart='True')
-        self.conf.config(group="haproxy_amphora", user_group='nogroup')
         self.conf.config(group="haproxy_amphora", base_path='/var/lib/octavia')
         self.conf.config(group="haproxy_amphora", bind_host='0.0.0.0')
         self.conf.config(group="haproxy_amphora", bind_port=9443)
@@ -65,8 +66,7 @@ class AgentJinjaTestCase(base.TestCase):
                            'haproxy_cmd = /usr/sbin/haproxy\n'
                            'respawn_count = 2\n'
                            'respawn_interval = 2\n'
-                           'use_upstart = True\n'
-                           'user_group = nogroup\n\n'
+                           'use_upstart = True\n\n'
                            '[health_manager]\n'
                            'controller_ip_port_list = 192.0.2.10:5555\n'
                            'heartbeat_interval = 10\n'
@@ -79,7 +79,10 @@ class AgentJinjaTestCase(base.TestCase):
                            'agent_server_network_dir = '
                            '/etc/network/interfaces.d/\n'
                            'agent_request_read_timeout = 120\n'
-                           'amphora_id = ' + AMP_ID)
+                           'amphora_id = ' + AMP_ID + '\n'
+                           'amphora_udp_driver = keepalived_lvs\n\n'
+                           '[controller_worker]\n'
+                           'loadbalancer_topology = SINGLE')
         agent_cfg = ajc.build_agent_config(AMP_ID)
         self.assertEqual(expected_config, agent_cfg)
 
@@ -98,8 +101,7 @@ class AgentJinjaTestCase(base.TestCase):
                            'haproxy_cmd = /usr/sbin/haproxy\n'
                            'respawn_count = 2\n'
                            'respawn_interval = 2\n'
-                           'use_upstart = False\n'
-                           'user_group = nogroup\n\n'
+                           'use_upstart = False\n\n'
                            '[health_manager]\n'
                            'controller_ip_port_list = 192.0.2.10:5555\n'
                            'heartbeat_interval = 10\n'
@@ -114,6 +116,45 @@ class AgentJinjaTestCase(base.TestCase):
                            'agent_server_network_file = '
                            '/etc/network/interfaces\n'
                            'agent_request_read_timeout = 120\n'
-                           'amphora_id = ' + AMP_ID)
+                           'amphora_id = ' + AMP_ID + '\n'
+                           'amphora_udp_driver = keepalived_lvs\n\n'
+                           '[controller_worker]\n'
+                           'loadbalancer_topology = SINGLE')
+        agent_cfg = ajc.build_agent_config(AMP_ID)
+        self.assertEqual(expected_config, agent_cfg)
+
+    def test_build_agent_config_with_new_udp_driver(self):
+        ajc = agent_jinja_cfg.AgentJinjaTemplater()
+        self.conf.config(group='amphora_agent',
+                         agent_server_network_file=None)
+        self.conf.config(group="amphora_agent",
+                         amphora_udp_driver='new_udp_driver')
+        expected_config = ('\n[DEFAULT]\n'
+                           'debug = False\n\n'
+                           '[haproxy_amphora]\n'
+                           'base_cert_dir = /var/lib/octavia/certs\n'
+                           'base_path = /var/lib/octavia\n'
+                           'bind_host = 0.0.0.0\n'
+                           'bind_port = 9443\n'
+                           'haproxy_cmd = /usr/sbin/haproxy\n'
+                           'respawn_count = 2\n'
+                           'respawn_interval = 2\n'
+                           'use_upstart = True\n\n'
+                           '[health_manager]\n'
+                           'controller_ip_port_list = 192.0.2.10:5555\n'
+                           'heartbeat_interval = 10\n'
+                           'heartbeat_key = TEST\n\n'
+                           '[amphora_agent]\n'
+                           'agent_server_ca = '
+                           '/etc/octavia/certs/client_ca.pem\n'
+                           'agent_server_cert = '
+                           '/etc/octavia/certs/server.pem\n'
+                           'agent_server_network_dir = '
+                           '/etc/network/interfaces.d/\n'
+                           'agent_request_read_timeout = 120\n'
+                           'amphora_id = ' + AMP_ID + '\n'
+                           'amphora_udp_driver = new_udp_driver\n\n'
+                           '[controller_worker]\n'
+                           'loadbalancer_topology = SINGLE')
         agent_cfg = ajc.build_agent_config(AMP_ID)
         self.assertEqual(expected_config, agent_cfg)
